# R package: minfi
This is the developer version of Bioconductor package [minfi](http://bioconductor.org/packages/devel/bioc/html/minfi.html).  Install in R as:

```r
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")
BiocManager::install('minfi')
```

## Software status

| Resource:     | Bioconductor        | Travis CI     |
| ------------- | ------------------- | ------------- |
| _Platforms:_  | _Multiple_          | _Linux_       |
| R CMD check   | <a href="http://bioconductor.org/checkResults/release/bioc-LATEST/minfi/"><img border="0" src="http://bioconductor.org/shields/build/release/bioc/minfi.svg" alt="Build status"></a> (release)</br><a href="http://bioconductor.org/checkResults/devel/bioc-LATEST/minfi/"><img border="0" src="http://bioconductor.org/shields/build/devel/bioc/minfi.svg" alt="Build status"></a> (devel) | <a href="https://travis-ci.org/kasperdanielhansen/minfi"><img src="https://travis-ci.org/kasperdanielhansen/minfi.svg" alt="Build status"></a> |
| Test coverage |                     | <a href="https://codecov.io/github/kasperdanielhansen/minfi?branch=master"><img src="https://codecov.io/github/kasperdanielhansen/minfi/coverage.svg?branch=master" alt="Coverage Status"/></a>   |                  |
