# Exported functions -----------------------------------------------------------

read.450k <- function(basenames, extended = FALSE, verbose = FALSE) {
    .Defunct("read.metharray")
}

read.450k.sheet <- function(base, pattern = "csv$", ignore.case = TRUE,
                            recursive = TRUE, verbose = TRUE) {
    .Defunct("read.metharray.sheet")
}


read.450k.exp <- function(base = NULL, targets = NULL, extended = FALSE,
                          recursive = FALSE, verbose = FALSE) {
    .Defunct("read.metharray.exp")
}
